=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2006-12-01 19:41:16 +0000
+++ bzrlib/commit.py	2006-12-22 03:28:50 +0000
@@ -212,6 +212,9 @@
         else:
             self.work_tree = working_tree
             self.branch = self.work_tree.branch
+            if getattr(self.work_tree, 'requires_rich_root', lambda: False)():
+                if not self.branch.repository.supports_rich_root():
+                    raise errors.RootNotRich()
         if message_callback is None:
             if message is not None:
                 if isinstance(message, str):

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2006-12-22 02:00:06 +0000
+++ bzrlib/tests/test_workingtree.py	2006-12-22 03:28:50 +0000
@@ -220,6 +220,55 @@
         self.assertEqual([], tree.get_parent_ids())
 
 
+class TestWorkingTreeFormat4(TestCaseWithTransport):
+    """Tests specific to WorkingTreeFormat3."""
+
+    def test_disk_layout(self):
+        tree = self.make_branch_and_tree('.', format=bzrdir.get_knit3_format())
+        control = tree.bzrdir
+        # we want:
+        # format 'Bazaar-NG Working Tree format 4'
+        # inventory = 1 entry for root
+        # pending-merges = ''
+        # no inventory.basis yet
+        t = control.get_workingtree_transport(None)
+        self.assertEqualDiff('Bazaar-NG Working Tree format 4',
+                             t.get('format').read())
+        self.assertContainsRe(t.get('inventory').read(), 
+                              '<inventory format="7">\n'
+                              '<directory file_id="[^"]*" name="" />\n'
+                              '</inventory>\n',
+                             )
+        self.assertEqualDiff('### bzr hashcache v5\n',
+                             t.get('stat-cache').read())
+        self.assertFalse(t.has('basis-inventory-cache'))
+        # no last-revision file means 'None' or 'NULLREVISION'
+        self.assertFalse(t.has('last-revision'))
+        tree.set_root_id('my-root-id')
+        tree.commit('test', rev_id='revision-1')
+        self.assertTrue(t.has('basis-inventory-cache'))
+        self.assertTrue(t.has('last-revision'))
+        self.assertEqualDiff(t.get('basis-inventory-cache').read(), 
+            '<inventory format="7" revision_id="revision-1">\n'
+            '<directory file_id="my-root-id" name="" revision="revision-1" />\n'
+            '</inventory>\n')
+    
+    def test_incompatible_repo(self):
+        control = bzrdir.get_knit1_format()
+        control.workingtree_format = workingtree.WorkingTreeFormat4()
+        tree = self.make_branch_and_tree('.', format=control)
+        self.assertRaises(errors.RootNotRich, tree.commit)
+
+    def test_compatible_repo(self):
+        tree = self.make_branch_and_tree('.', format=bzrdir.get_knit3_format())
+        tree.set_root_id('my-root-id')
+        tree.commit('test', rev_id='revision-1')
+        tree.commit('test', rev_id='revision-2')
+        revision_tree = tree.branch.repository.revision_tree('revision-2')
+        self.assertEqual('revision-1', 
+                         revision_tree.inventory['my-root-id'].revision)
+
+
 class TestFormat2WorkingTree(TestCaseWithTransport):
     """Tests that are specific to format 2 trees."""
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-12-22 02:00:06 +0000
+++ bzrlib/workingtree.py	2006-12-22 03:28:50 +0000
@@ -309,6 +309,9 @@
         self._control_files.break_lock()
         self.branch.break_lock()
 
+    def requires_rich_root(self):
+        return self._format.requires_rich_root
+
     def _set_inventory(self, inv, dirty):
         """Set the internal cached inventory.
 
@@ -1518,7 +1521,7 @@
     def _create_basis_xml_from_inventory(self, revision_id, inventory):
         """Create the text that will be saved in basis-inventory"""
         inventory.revision_id = revision_id
-        return xml6.serializer_v6.write_inventory_to_string(inventory)
+        return xml7.serializer_v7.write_inventory_to_string(inventory)
 
     def _cache_basis_inventory(self, new_revision):
         """Cache new_revision as the basis inventory."""
@@ -1539,7 +1542,7 @@
             xml = self.branch.repository.get_inventory_xml(new_revision)
             firstline = xml.split('\n', 1)[0]
             if (not 'revision_id="' in firstline or 
-                'format="6"' not in firstline):
+                'format="7"' not in firstline):
                 inv = self.branch.repository.deserialise_inventory(
                     new_revision, xml)
                 xml = self._create_basis_xml_from_inventory(new_revision, inv)
@@ -2037,6 +2040,8 @@
     _formats = {}
     """The known formats."""
 
+    requires_rich_root = False
+
     @classmethod
     def find_format(klass, a_bzrdir):
         """Return the format for the working tree object in a_bzrdir."""
@@ -2278,6 +2283,8 @@
 
     _tree_class = WorkingTree4
 
+    requires_rich_root = True
+
     def get_format_string(self):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar-NG Working Tree format 4"

